package co.ab180.airbridge.flutter;

import android.content.Context;
import android.content.Intent;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import io.airbridge.AirBridge;
import io.airbridge.State;
import io.airbridge.deeplink.DeepLink;
import io.flutter.plugin.common.PluginRegistry;

public class AirbridgeFL {

    //
    // flutter plugin setting
    //

    public static void registerWith(PluginRegistry.Registrar registrar) {
        DeeplinkAPI.registerWith(registrar);
        EventAPI.registerWith(registrar);
        StateAPI.registerWith(registrar);
        SettingAPI.registerWith(registrar);
        PlacementAPI.registerWith(registrar);
    }

    //
    // init
    //

    /**
     * Initialize Airbridge SDK
     * <p>
     * You should call this method on MainApplication#onCreate
     *
     * @param   context
     *              context from MainApplication#onCreate
     * @param   appName
     *              App Name in English
     * @param   appToken
     *              App Token
     */
    public static void init(@Nonnull final Context context, @Nonnull String appName, @Nonnull String appToken) {
        AirBridge.init(context, appName, appToken);
        DeeplinkAPI.setInitialDeeplinkCallback();
    }

    //
    // interface
    //

    /**
     * Return singleton instance of DeepLink
     * <p>
     * you can give deeplink information to Airbridge SDK with this instance
     */
    public static DeepLink getDeeplink() {
        return AirBridge.getDeeplink();
    }

    //
    // setting
    //

    /**
     * Set timeout of session
     * @param   millisecond
     *              amount of timeout in millisecond
     */
    public static void setSessionTimeout(long millisecond) {
        AirBridge.setCustomSessionTimeOut(millisecond / 1000);
    }

    /**
     * Set timeout of deeplink-fetch
     * @param   millisecond
     *              amount of timeout in millisecond
     */
    public static void setDeeplinkFetchTimeout(int millisecond) {
        AirBridge.getDeeplink().setHandleTrackingLinkTimeout(millisecond);
    }

    /**
     * Set is Airbridge SDK hash user-infomation before send to server
     * <p>
     * default value is YES
     * @param   enable
     *              hash user-infomation or not
     */
    public static void setIsUserInfoHashed(boolean enable) {
        AirBridge.getTracker().setIsUserInfoHashed(enable);
    }

    /**
     * Set is Airbridge SDK track airbridge-deeplink only
     * <p>
     * default value is YES
     * @param   enable
     *              track airbridge-deeplink only or not
     */
    public static void setIsTrackAirbridgeDeeplinkOnly(boolean enable) {
        if(enable) {
            DeepLink.trackAirbridgeLinkOnly();
        }
    }

    /**
     * Set is enable tracking
     * <p>
     * default value is YES
     * @param   enable
     *              enable tracking or not
     */
    public static void setIsTrackingEnabled(boolean enable) {
        AirBridge.setLimitUserTracking(!enable);
    }

    //
    // deeplink
    //

    /**
     * Give deeplink infomation to Airbridge SDK
     * @param	intent
     *				intent which contain deeplink information
     */
    public static void onNewIntent(@Nullable Intent intent) {
        AirBridge.getTracker().onNewIntent(intent);
    }
}
