package co.ab180.airbridge.flutter;

import android.os.Handler;
import android.os.Looper;

import io.airbridge.AirBridge;
import io.airbridge.deeplink.DeepLink;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.view.FlutterNativeView;

public class DeeplinkAPI implements EventChannel.StreamHandler {

    private static String initialDeeplink;
    private static String initialDeferredDeeplink;

    static void setInitialDeeplinkCallback() {
        AirBridge.getDeeplink().setDeeplinkCallback(new DeepLink.Callback() {
            @Override
            public void TrackingLinkResponse(String URL) {
                initialDeeplink = URL;
            }
        });
        AirBridge.getDeeplink().setDeferredDeeplinkCallback(new DeepLink.Callback() {
            @Override
            public void TrackingLinkResponse(String URL) {
                initialDeferredDeeplink = URL;
            }
        });
    }

    private Handler mainHandler = new Handler(Looper.getMainLooper());

    //
    // flutter plugin setting
    //

    public static void registerWith(PluginRegistry.Registrar registrar) {
        final DeeplinkAPI deeplinkAPI = new DeeplinkAPI();

        final EventChannel deeplinkEvent = new EventChannel(
                registrar.messenger(),
                "airbridge_flutter_sdk/event/deeplink"
        );
        deeplinkEvent.setStreamHandler(deeplinkAPI);
        final EventChannel deferredDeeplinkEvent = new EventChannel(
                registrar.messenger(),
                "airbridge_flutter_sdk/event/deferred_deeplink"
        );
        deferredDeeplinkEvent.setStreamHandler(deeplinkAPI);

        registrar.addViewDestroyListener(new PluginRegistry.ViewDestroyListener() {
            @Override
            public boolean onViewDestroy(FlutterNativeView flutterNativeView) {
                setInitialDeeplinkCallback();
                return false;
            }
        });
    }

    @Override
    public void onListen(Object o, final EventChannel.EventSink eventSink) {
        String target = Get.type(String.class, o);
        if (target == null || eventSink == null) {
            return;
        }

        if (target.equals("deeplink")) {
            if (initialDeeplink != null) {
                mainHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        eventSink.success(initialDeeplink);
                        initialDeeplink = null;
                    }
                });
            }

            AirBridge.getDeeplink().setDeeplinkCallback(new DeepLink.Callback() {
                @Override
                public void TrackingLinkResponse(final String URL) {
                    mainHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            eventSink.success(URL);
                        }
                    });
                }
            });
        } else if (target.equals("deferred_deeplink")) {
            if (initialDeferredDeeplink != null) {
                mainHandler.post(new Runnable() {
                    @Override
                    public void run() {
                        eventSink.success(initialDeferredDeeplink);
                        initialDeferredDeeplink = null;
                    }
                });
            }

            AirBridge.getDeeplink().setDeferredDeeplinkCallback(new DeepLink.Callback() {
                @Override
                public void TrackingLinkResponse(final String URL) {
                    mainHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            eventSink.success(URL);
                        }
                    });
                }
            });
        }
    }

    @Override
    public void onCancel(Object o) {
        String target = Get.type(String.class, o);
        if (target == null) {
            return;
        }

        if (target.equals("deeplink")) {
            AirBridge.getDeeplink().setDeeplinkCallback(new DeepLink.Callback() {
                @Override
                public void TrackingLinkResponse(String URL) {}
            });
        } else if (target.equals("deferred_deeplink")) {
            AirBridge.getDeeplink().setDeferredDeeplinkCallback(new DeepLink.Callback() {
                @Override
                public void TrackingLinkResponse(String URL) {}
            });
        }
    }
}