package co.ab180.airbridge.flutter;

import java.util.ArrayList;
import java.util.HashMap;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import io.airbridge.AirBridge;
import io.airbridge.ecommerce.AddedToCartEvent;
import io.airbridge.ecommerce.HomeViewEvent;
import io.airbridge.ecommerce.Product;
import io.airbridge.ecommerce.ProductDetailsViewEvent;
import io.airbridge.ecommerce.ProductListViewEvent;
import io.airbridge.ecommerce.PurchaseEvent;
import io.airbridge.ecommerce.SearchResultViewEvent;
import io.airbridge.statistics.events.GoalEvent;
import io.airbridge.statistics.events.inapp.SignInEvent;
import io.airbridge.statistics.events.inapp.SignOutEvent;
import io.airbridge.statistics.events.inapp.SignUpEvent;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;

public class EventAPI implements MethodChannel.MethodCallHandler {

    //
    // flutter plugin setting
    //

    public static void registerWith(PluginRegistry.Registrar registrar) {
        final MethodChannel channel = new MethodChannel(
                registrar.messenger(),
                "airbridge_flutter_sdk/method/event"
        );
        channel.setMethodCallHandler(new EventAPI());
    }

    @Override
    public void onMethodCall(MethodCall call, MethodChannel.Result result) {
        HashMap arguments = Get.type(HashMap.class, call.arguments);

        switch (call.method) {
            case "sendSignIn":
                sendSignIn(arguments, result);
                break;
            case "sendSignUp":
                sendSignUp(arguments, result);
                break;
            case "sendSignOut":
                sendSignOut(arguments, result);
                break;
            case "sendViewHome":
                sendViewHome(arguments, result);
                break;
            case "sendViewSearchResult":
                sendViewSearchResult(arguments, result);
                break;
            case "sendViewProductList":
                sendViewProductList(arguments, result);
                break;
            case "sendViewProductDetail":
                sendViewProductDetail(arguments, result);
                break;
            case "sendAddToCart":
                sendAddToCart(arguments, result);
                break;
            case "sendPurchase":
                sendPurchase(arguments, result);
                break;
            case "sendCustom":
                sendCustom(arguments, result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    //
    // flutter method
    //

    private void sendSignIn(HashMap arguments, MethodChannel.Result result) {
        HashMap user = Get.type(HashMap.class, arguments, "user");
        HashMap option = Get.type(HashMap.class, arguments, "option");

        String id = Get.type(String.class, user, "id");
        String email = Get.type(String.class, user, "email");
        String phone = Get.type(String.class, user, "phone");

        SignInEvent event = new SignInEvent();

        addOptionToEvent(event, option);
        event.setUserId(id);
        event.setUserEmail(email);
        event.setUserPhoneNumber(phone);

        AirBridge.getTracker().sendEvent(event);

        result.success(null);
    }

    private void sendSignUp(HashMap arguments, MethodChannel.Result result) {
        HashMap user = Get.type(HashMap.class, arguments, "user");
        HashMap option = Get.type(HashMap.class, arguments, "option");

        String id = Get.type(String.class, user, "id");
        String email = Get.type(String.class, user, "email");
        String phone = Get.type(String.class, user, "phone");

        SignUpEvent event = new SignUpEvent();

        addOptionToEvent(event, option);
        event.setUserId(id);
        event.setUserEmail(email);
        event.setUserPhoneNumber(phone);

        AirBridge.getTracker().sendEvent(event);

        result.success(null);
    }

    private void sendSignOut(HashMap arguments, MethodChannel.Result result) {
        HashMap option = Get.type(HashMap.class, arguments, "option");

        SignOutEvent event = new SignOutEvent();

        addOptionToEvent(event, option);

        AirBridge.getTracker().sendEvent(event);

        result.success(null);
    }

    private void sendViewHome(HashMap arguments, MethodChannel.Result result) {
        HashMap option = Get.type(HashMap.class, arguments, "option");

        HomeViewEvent event = new HomeViewEvent();

        addOptionToEvent(event, option);

        AirBridge.getTracker().sendEvent(event);

        result.success(null);
    }

    private void sendViewSearchResult(HashMap arguments, MethodChannel.Result result) {
        HashMap option = Get.type(HashMap.class, arguments, "option");

        String query = Get.type(String.class, arguments, "query");
        ArrayList products = Get.type(ArrayList.class, arguments, "products");
        ArrayList<Product> nativeProducts = convertProducts(products);

        SearchResultViewEvent event = new SearchResultViewEvent();

        addOptionToEvent(event, option);
        event.setQuery(query);
        event.addProducts(nativeProducts);

        AirBridge.getTracker().sendEvent(event);

        result.success(null);
    }

    private void sendViewProductList(HashMap arguments, MethodChannel.Result result) {
        HashMap option = Get.type(HashMap.class, arguments, "option");

        String listID = Get.type(String.class, arguments, "listID");
        ArrayList products = Get.type(ArrayList.class, arguments, "products");
        ArrayList<Product> nativeProducts = convertProducts(products);

        ProductListViewEvent event = new ProductListViewEvent();

        addOptionToEvent(event, option);
        event.setListId(listID);
        event.addProducts(nativeProducts);

        AirBridge.getTracker().sendEvent(event);

        result.success(null);
    }

    private void sendViewProductDetail(HashMap arguments, MethodChannel.Result result) {
        HashMap option = Get.type(HashMap.class, arguments, "option");

        ArrayList products = Get.type(ArrayList.class, arguments, "products");
        ArrayList<Product> nativeProducts = convertProducts(products);

        ProductDetailsViewEvent event = new ProductDetailsViewEvent();

        addOptionToEvent(event, option);
        event.addProducts(nativeProducts);

        AirBridge.getTracker().sendEvent(event);

        result.success(null);
    }

    private void sendAddToCart(HashMap arguments, MethodChannel.Result result) {
        HashMap option = Get.type(HashMap.class, arguments, "option");

        String cartID = Get.type(String.class, arguments, "cartID");
        String currency = Get.type(String.class, arguments, "currency");
        Integer total = Get.type(Integer.class, Get.type(Number.class, arguments, "total"));
        ArrayList products = Get.type(ArrayList.class, arguments, "products");
        ArrayList<Product> nativeProducts = convertProducts(products);

        AddedToCartEvent event = new AddedToCartEvent();

        addOptionToEvent(event, option);
        event.setCartId(cartID);
        event.setCurrency(currency);
        event.setTotalValue(total);
        event.addProducts(nativeProducts);

        AirBridge.getTracker().sendEvent(event);

        result.success(null);
    }

    private void sendPurchase(HashMap arguments, MethodChannel.Result result) {
        HashMap option = Get.type(HashMap.class, arguments, "option");

        String transactionID = Get.type(String.class, arguments, "transactionID");
        Boolean isInAppPurchase = Get.type(Boolean.class, arguments, "isInAppPurchase");
        String currency = Get.type(String.class, arguments, "currency");
        Integer total = Get.type(Integer.class, Get.type(Number.class, arguments, "total"));
        ArrayList products = Get.type(ArrayList.class, arguments, "products");
        ArrayList<Product> nativeProducts = convertProducts(products);

        PurchaseEvent event = new PurchaseEvent();

        addOptionToEvent(event, option);
        event.setTransactionId(transactionID);
        event.setInAppPurchased(isInAppPurchase);
        event.setCurrency(currency);
        event.setTotalValue(total);
        event.addProducts(nativeProducts);

        AirBridge.getTracker().sendEvent(event);

        result.success(null);
    }

    private void sendCustom(HashMap arguments, MethodChannel.Result result) {
        String category = Get.type(String.class, arguments, "category");
        HashMap option = Get.type(HashMap.class, arguments, "option");

        GoalEvent event = new GoalEvent(category);
        
        addOptionToEvent(event, option);

        AirBridge.getTracker().sendEvent(event);

        result.success(null);
    }

    //
    // private - tool
    //

    /**
     * Add option infomation to event
     * <p>
     * add null when some data has unmatch type
     * @param       event
     *                  target event
     * @param       option
     *                  json-object which has option infomation
     */
    private void addOptionToEvent(@Nonnull GoalEvent event, @Nullable HashMap option) {
        if (option == null) return;

        String action = Get.type(String.class, option, "action");
        String label = Get.type(String.class, option, "label");
        Integer value = Get.type(Integer.class, Get.type(Number.class, option, "value"));
        HashMap attributes = Get.type(HashMap.class, option, "attributes");

        event.setAction(action);
        event.setLabel(label);
        event.setValue(value);

        if (attributes != null) {
            for (Object _key : attributes.keySet()) {
                String key = Get.type(String.class, _key);

                event.setCustomAttribute(key, attributes.get(key));
            }
        }
    }

    /**
     * convert array-of-maps to array-of-Products
     * <p>
     * return null when inputs is null
     * <p>
     * add null when some data has unmatch type
     * @param       inputs
     *                  json-array of json-objects
     * @return		array of Products
     */
    @Nullable
    private ArrayList<Product> convertProducts(@Nullable ArrayList inputs) {
        if (inputs == null) return null;

        ArrayList<Product> outputs = new ArrayList<>();

        for (int i=0; i<inputs.size(); i++) {
            HashMap object = Get.type(HashMap.class, inputs, i);

            Product product = convertProduct(object);

            outputs.add(product);
        }

        return outputs;
    }

    /**
     * convert map to Product
     * <p>
     * return null when input is null
     * <p>
     * add null when some data has unmatch type
     * @param       input
     *                  json-object which has product infomation
     * @return		Product
     */
    @Nullable
    private Product convertProduct(@Nullable HashMap input) {
        if (input == null) return null;

        Product output = new Product();

        String id = Get.type(String.class, input, "id");
        String name = Get.type(String.class, input, "name");
        String currency = Get.type(String.class, input, "currency");
        Integer quantity = Get.type(Integer.class, Get.type(Number.class, input, "quantity"));
        Float price = Get.type(Float.class, Get.type(Number.class, input, "price"));
        Integer position = Get.type(Integer.class, Get.type(Number.class, input, "position"));

        output.setProductId(id);
        output.setName(name);
        output.setCurrency(currency);
        output.setQuantity(quantity);
        output.setPrice(price);
        output.setPositionInList(position);

        return output;
    }
}
