package co.ab180.airbridge.flutter;

import java.util.HashMap;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;

public class SettingAPI implements MethodChannel.MethodCallHandler {

    //
    // flutter plugin setting
    //

    public static void registerWith(PluginRegistry.Registrar registrar) {
        final MethodChannel channel = new MethodChannel(
                registrar.messenger(),
                "airbridge_flutter_sdk/method/setting"
        );
        channel.setMethodCallHandler(new SettingAPI());
    }

    @Override
    public void onMethodCall(MethodCall call, MethodChannel.Result result) {
        HashMap arguments = Get.type(HashMap.class, call.arguments);

        if (call.method.equals("setSessionTimeout")) {
            setSessionTimeout(arguments, result);
        } else if (call.method.equals("setDeeplinkFetchTimeout")) {
            setDeeplinkFetchTimeout(arguments, result);
        } else if (call.method.equals("setIsUserInfoHashed")) {
            setIsUserInfoHashed(arguments, result);
        } else if (call.method.equals("setIsTrackAirbridgeDeeplinkOnly")) {
            setIsTrackAirbridgeDeeplinkOnly(arguments, result);
        } else if (call.method.equals("setIsTrackingEnabled")) {
            setIsTrackingEnabled(arguments, result);
        } else {
            result.notImplemented();
        }
    }

    //
    // flutter method
    //

    private void setSessionTimeout(HashMap arguments, MethodChannel.Result result) {
        Long timeout = Get.type(
                Long.class,
                Get.type(Number.class, arguments, "timeout")
        );

        AirbridgeFL.setSessionTimeout(timeout);

        result.success(null);
    }

    private void setDeeplinkFetchTimeout(HashMap arguments, MethodChannel.Result result) {
        Integer timeout = Get.type(
                Integer.class,
                Get.type(Number.class, arguments, "timeout")
        );

        AirbridgeFL.setDeeplinkFetchTimeout(timeout);

        result.success(null);
    }

    private void setIsUserInfoHashed(HashMap arguments, MethodChannel.Result result) {
        Boolean enable = Get.type(Boolean.class, arguments, "enable");

        AirbridgeFL.setIsUserInfoHashed(enable);

        result.success(null);
    }

    private void setIsTrackAirbridgeDeeplinkOnly(HashMap arguments, MethodChannel.Result result) {
        Boolean enable = Get.type(Boolean.class, arguments, "enable");

        AirbridgeFL.setIsTrackAirbridgeDeeplinkOnly(enable);

        result.success(null);
    }

    private void setIsTrackingEnabled(HashMap arguments, MethodChannel.Result result) {
        Boolean enable = Get.type(Boolean.class, arguments, "enable");

        AirbridgeFL.setIsTrackingEnabled(enable);

        result.success(null);
    }
}

