package co.ab180.airbridge.flutter;

import java.util.HashMap;

import io.airbridge.AirBridge;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;

public class StateAPI implements MethodChannel.MethodCallHandler {

    //
    // flutter plugin setting
    //

    public static void registerWith(PluginRegistry.Registrar registrar) {
        final MethodChannel channel = new MethodChannel(
                registrar.messenger(),
                "airbridge_flutter_sdk/method/state"
        );
        channel.setMethodCallHandler(new StateAPI());
    }

    @Override
    public void onMethodCall(MethodCall call, MethodChannel.Result result) {
        HashMap arguments = Get.type(HashMap.class, call.arguments);

        if (call.method.equals("setUser")) {
            setUser(arguments, result);
        } else if (call.method.equals("updateUser")) {
            updateUser(arguments, result);
        } else if (call.method.equals("getDeviceUUID")) {
            getDeviceUUID(result);
        } else if (call.method.equals("getIsLimitADTracking")) {
            getIsLimitADTracking(result);
        } else {
            result.notImplemented();
        }
    }

    //
    // flutter method
    //

    private void setUser(HashMap arguments, MethodChannel.Result result) {
        String ID = Get.type(String.class, arguments, "id");
        String email = Get.type(String.class, arguments, "email");
        String phone = Get.type(String.class, arguments, "phone");

        AirBridge.getTracker().setUserId(ID);
        AirBridge.getTracker().setUserEmail(email);
        AirBridge.getTracker().setUserPhoneNumber(phone);

        result.success(null);
    }

    private void updateUser(HashMap arguments, MethodChannel.Result result) {
        String ID = Get.type(String.class, arguments, "id");
        String email = Get.type(String.class, arguments, "email");
        String phone = Get.type(String.class, arguments, "phone");

        if (ID != null) {
            AirBridge.getTracker().setUserId(ID);
        }
        if (email != null) {
            AirBridge.getTracker().setUserEmail(email);
        }
        if (phone != null) {
            AirBridge.getTracker().setUserPhoneNumber(phone);
        }

        result.success(null);
    }

    private void getDeviceUUID(final MethodChannel.Result result) {
        AirBridge.getDeviceUUID(new AirBridge.UUIDCallback() {
            @Override
            public void done(String uuid) {
                result.success(uuid);
            }
        });
    }

    private void getIsLimitADTracking(final MethodChannel.Result result) {
        AirBridge.isLimitADTracking(new AirBridge.ADIDCallback() {
            @Override
            public void done(Boolean isLAT) {
                result.success(isLAT);
            }
        });
    }
}
