import 'package:airbridge_flutter_sdk/util/log.dart';
import 'package:flutter/services.dart';

class DeeplinkAPI {
    final EventChannel _deeplinkEvent;
    final EventChannel _deferredDeeplinkEvent;

    Function _deeplinkListener;
    Function _deferredDeepinkListener;

    String _initialDeeplink;
    String _initialDeferredDeeplink;

    DeeplinkAPI(this._deeplinkEvent, this._deferredDeeplinkEvent)
    {
        _deeplinkListener = (deeplink) {
            _initialDeeplink = deeplink;
        };
        _deferredDeepinkListener = (deeplink) {
            _initialDeferredDeeplink = deeplink;
        };

        _deeplinkEvent
            .receiveBroadcastStream("deeplink")
            .listen((deeplink) {
                Log.info('$deeplink [deeplink]');

                _deeplinkListener(deeplink);
            });
        _deferredDeeplinkEvent
            .receiveBroadcastStream("deferred_deeplink")
            .listen((deeplink) {
                Log.info('[deferred deeplink] \n $deeplink');

                _deferredDeepinkListener(deeplink);
            });
    }

    void setDeeplinkListener(Function listener) {
        Log.info('set deeplink listener');

        if (_initialDeeplink != null) {
            listener(_initialDeeplink);
            _initialDeeplink = null;
        }

        this._deeplinkListener = listener;
    }

    void setDeferredDeeplinkListener(Function listener) {
        Log.info('set deferred deeplink listener');

        if (_initialDeferredDeeplink != null) {
            listener(_initialDeferredDeeplink);
            _initialDeferredDeeplink = null;
        }

        this._deferredDeepinkListener = listener;
    }
}
